#include <unity.h>

#include "kbf/net.h"

using namespace kbf;

TEST_CASE("MAC address functions", "[kbf_net]") {
    uint8_t addr[] = {0, 0, 11, 128, 176, 255};

    auto mac = net::MAC(addr);
    TEST_ASSERT_EQUAL_STRING_MESSAGE("00:00:0B:80:B0:FF", mac.str().c_str(), "ctor / str()");

    net::setMac(mac);

    TEST_ASSERT_EQUAL_STRING_MESSAGE(mac.str().c_str(), net::getMac().str().c_str(), "getMac");
}

TEST_CASE("IP address functions", "[kbf_net]") {
    esp_ip4_addr testAddr = {2952898752};
    static const char *const testIP = "192.168.1.176";

    auto ipFromAddr = net::IP(testAddr);
    TEST_ASSERT_TRUE(ipFromAddr)
    TEST_ASSERT_EQUAL_STRING(testIP, ipFromAddr.str().c_str());
    TEST_ASSERT_EQUAL(0  , ipFromAddr[-1]);
    TEST_ASSERT_EQUAL(192, ipFromAddr[0]);
    TEST_ASSERT_EQUAL(168, ipFromAddr[1]);
    TEST_ASSERT_EQUAL(1  , ipFromAddr[2]);
    TEST_ASSERT_EQUAL(176, ipFromAddr[3]);
    TEST_ASSERT_EQUAL(0  , ipFromAddr[4]);

    auto ipFromString = net::IP(testIP);
    TEST_ASSERT_TRUE(ipFromString)
    TEST_ASSERT_EQUAL_STRING(testIP, ipFromString.str().c_str());

    TEST_ASSERT_TRUE_MESSAGE(ipFromAddr == ipFromString, "operator==")
    TEST_ASSERT_EQUAL_STRING(ipFromAddr.str().c_str(), ipFromString.str().c_str());

    for (const auto &invalidIP : {"", "invalid", "255.255.255.256", "123.123.123.", "0.0.-1.0"}) {
        auto ip = net::IP(invalidIP);
        TEST_ASSERT_FALSE(ip)
        TEST_ASSERT_EQUAL_STRING("", ip.str().c_str());
    }

    for (const auto &validIp : {"0.0.0.0", "255.255.255.255"}) {
        auto ip = net::IP(validIp);
        TEST_ASSERT_TRUE(ip)
        TEST_ASSERT_EQUAL_STRING(validIp, ip.str().c_str());
    }
}
